<?php

namespace Drupal\mason;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\blazy\BlazyManagerBase;
use Drupal\mason\Entity\Mason;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Implements MasonManagerInterface.
 *
 * @todo lazyload[x], granular image styles, decent skins, stamps.
 */
class MasonManager extends BlazyManagerBase implements MasonManagerInterface, TrustedCallbackInterface {

  /**
   * The mason skin manager service.
   *
   * @var \Drupal\mason\MasonSkinManagerInterface
   */
  protected $skinManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->setSkinManager($container->get('mason.skin_manager'));
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return ['preRenderMason'];
  }

  /**
   * Returns mason skin manager service.
   */
  public function skinManager() {
    return $this->skinManager;
  }

  /**
   * Sets mason skin manager service.
   */
  public function setSkinManager(MasonSkinManagerInterface $skin_manager) {
    $this->skinManager = $skin_manager;
    return $this;
  }

  /**
   * Returns array of needed assets suitable for #attached for the given mason.
   */
  public function attach(array $attach = []) {
    $attach['blazy'] = TRUE;
    $load = parent::attach($attach);

    $load['library'][] = 'mason/load';

    if (isset($attach['skin']) && $skin = $attach['skin']) {
      $skins = $this->skinManager->getSkins();
      $provider = isset($skins[$skin]['provider']) ? $skins[$skin]['provider'] : 'mason';
      $load['library'][] = 'mason/' . $provider . '.' . $skin;
    }

    $load['drupalSettings']['mason'] = Mason::load('default')->getOptions();

    $this->moduleHandler->alter('mason_attach', $load, $attach);
    return $load;
  }

  /**
   * {@inheritdoc}
   */
  public function build(array $build = []) {
    foreach (MasonDefault::themeProperties() as $key) {
      $build[$key] = isset($build[$key]) ? $build[$key] : [];
    }

    $mason = [
      '#theme'      => 'mason',
      '#items'      => [],
      '#build'      => $build,
      '#pre_render' => [[$this, 'preRenderMason']],
      'items'       => [],
    ];

    $this->moduleHandler->alter('mason_build', $mason);
    return empty($build['items']) ? [] : $mason;
  }

  /**
   * {@inheritdoc}
   */
  public function buildItems(array $build) {
    $settings = $build['settings'];
    $items    = [];

    foreach ($build['items'] as $delta => $item) {
      if ($delta > $settings['_count'] - 1) {
        continue;
      }

      $attributes = isset($item['attributes']) ? $item['attributes'] : [];
      $settings   = isset($item['settings']) ? array_merge($settings, $item['settings']) : $settings;

      $content = [
        'box'     => isset($item['box']) ? $item['box'] : [],
        'caption' => isset($item['caption']) ? $item['caption'] : [],
      ];

      $box = [
        '#theme'      => 'mason_box',
        '#item'       => $content,
        '#delta'      => $delta,
        '#attributes' => $attributes,
        '#settings'   => $settings,
        '#cache'      => isset($item['#cache']) ? $item['#cache'] : [],
      ];

      $items[] = $box;
      unset($box);
    }

    return $items;
  }

  /**
   * {@inheritdoc}
   */
  public function preRenderMason($element) {
    $build = $element['#build'];
    unset($element['#build']);

    // Build mason elements.
    $settings    = &$build['settings'];
    $settings   += MasonDefault::htmlSettings();
    $attachments = $this->attach($settings);
    $attachments = empty($build['attached']) ? $attachments : NestedArray::mergeDeep($build['attached'], $attachments);
    $cache       = $this->getCacheMetadata($build);
    $optionset   = $build['optionset'] ?: Mason::loadWithFallback($settings['optionset']);

    $settings['_count'] = count($optionset->getOption('sizes'));
    unset($build['optionset']);

    $element['#optionset'] = $optionset;
    $element['#settings']  = $settings;
    $element['#items']     = $this->buildItems($build);
    $element['#json']      = $optionset->getJson();
    $element['#attached']  = empty($element['#attached']) ? $attachments : NestedArray::mergeDeep($element['#attached'], $attachments);
    $element['#cache']     = empty($element['#cache']) ? $cache : NestedArray::mergeDeep($element['#cache'], $cache);

    unset($build);
    return $element;
  }

}
