<?php

namespace Drupal\mason\Plugin\views\style;

use Drupal\Core\Form\FormStateInterface;
use Drupal\blazy\Dejavu\BlazyStylePluginBase;
use Drupal\mason\MasonDefault;
use Drupal\mason\Entity\Mason;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Mason style plugin.
 *
 * @ingroup views_style_plugins
 *
 * @ViewsStyle(
 *   id = "mason",
 *   title = @Translation("Mason"),
 *   help = @Translation("Display the results in a Mason."),
 *   theme = "mason",
 *   register_theme = FALSE,
 *   display_types = {"normal"}
 * )
 */
class MasonViews extends BlazyStylePluginBase {

  /**
   * The mason service manager.
   *
   * @var \Drupal\mason\MasonManagerInterface
   */
  protected $manager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->manager = $container->get('mason.manager');
    $instance->blazyManager = isset($instance->blazyManager) ? $instance->blazyManager : $container->get('blazy.manager');

    return $instance;
  }

  /**
   * Returns the mason manager.
   */
  public function manager() {
    return $this->manager;
  }

  /**
   * Returns the mason admin.
   */
  public function admin() {
    return \Drupal::service('mason.admin');
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = ['fillers' => ['default' => ''], 'stamp' => ['default' => []]];
    foreach (MasonDefault::extendedSettings() as $key => $value) {
      $options[$key] = ['default' => $value];
    }
    return $options + parent::defineOptions();
  }

  /**
   * Overrides parent::buildOptionsForm().
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    $fields = MasonDefault::viewFieldOptions();
    $definition = $this->getDefinedFieldOptions($fields);
    $definition['opening_class'] = 'form--views';
    $definition['_views'] = TRUE;
    $count = empty($definition['captions']) ? 0 : count($definition['captions']);
    $definition['captions_count'] = $count;

    $this->admin()->buildSettingsForm($form, $definition);

    $title = '<p class="form__header form__title">';
    $title .= $this->t('Check Vanilla if using content/custom markups, not fields. <small>See it under <strong>Format > Show</strong> section. Otherwise Gridstack markups apply which require some fields added below.</small>');
    $title .= '</p>';

    $form['opening']['#markup'] .= $title;
    $form['image']['#description'] .= ' ' . $this->t('Be sure to UNCHECK "Use field template" (by default already UNCHECKED) to have it work for Blazy lazyloading. Use Blazy formatters for relevant features such as Colorbox/Photobox/Photoswipe, etc., or multimedia supports.');
  }

  /**
   * {@inheritdoc}
   */
  protected function buildSettings() {
    // @todo move it into self::prepareSettings() post blazy:2.x.
    $this->options['item_id'] = 'box';
    $this->options['namespace'] = 'mason';
    $settings = parent::buildSettings();
    $settings += MasonDefault::entitySettings();

    // Prepare needed settings to work with.
    $settings['caption']  = array_filter($settings['caption']);
    $settings['ratio']    = '';
    $settings['_noratio'] = TRUE;

    return $settings;
  }

  /**
   * Overrides StylePluginBase::render().
   *
   * @todo use self::buildSettings() post blazy:1.x update.
   */
  public function render() {
    $settings = $this->buildSettings();
    $optionset = Mason::load($settings['optionset']);

    $elements = [];
    foreach ($this->renderGrouping($this->view->result, $settings['grouping']) as $rows) {
      $element = $this->buildElements($settings, $rows);

      // Supports Blazy multi-breakpoint images if using Blazy formatter.
      $settings['first_image'] = isset($rows[0]) ? $this->getFirstImage($rows[0]) : [];

      // Supports Blazy formatter multi-breakpoint images if available.
      if (isset($element[0])) {
        $this->manager->isBlazy($settings, $element[0]);
      }

      $build = [
        'items'     => $element,
        'optionset' => $optionset,
        'settings'  => $settings,
      ];

      $elements = $this->manager->build($build);
    }

    return $elements;
  }

  /**
   * Returns mason contents.
   */
  public function buildElements(array $settings, $rows) {
    $build = [];
    $item_id = $settings['item_id'];

    foreach ($rows as $index => $row) {
      $this->view->row_index = $index;

      $box             = [];
      $box['delta']    = $index;
      $box[$item_id]   = [];
      $box['settings'] = $settings;

      if (!empty($settings['class'])) {
        $classes = $this->getFieldString($row, $settings['class'], $index);
        $box['settings']['class'] = empty($classes[$index]) ? [] : $classes[$index];
      }

      // Use Vanilla mason if so configured, ignoring Mason markups.
      if (!empty($settings['vanilla'])) {
        $box[$item_id] = $this->view->rowPlugin->render($row);
      }
      else {
        // Build individual row/element contents.
        $this->buildElement($box, $row, $index);
      }

      // Build mason items.
      $build[] = $box;
      unset($box);
    }

    unset($this->view->row_index);
    return $build;
  }

}
